@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2020 Linux Studio Plugins Project <https://lsp-plug.in/>
- *           (C) 2020 Vladimir Sadovnikov <sadko4u@gmail.com>
+ * Copyright (C) 2023 Linux Studio Plugins Project <https://lsp-plug.in/>
+ *           (C) 2023 Vladimir Sadovnikov <sadko4u@gmail.com>
  *
  * This file is part of lsp-plugins
  * Created on: 03 авг. 2016 г.
@@ -625,5 +625,5 @@ namespace lsp
             v->write("vHpfBuf", vHpfBuf);
             v->write("pData", pData);
         }
-    }
+    } /* namespace dspu */
 } /* namespace lsp */
