@@ -25,7 +25,7 @@
 
 #define LSP_PLUGINS_COMP_DELAY_VERSION_MAJOR         1
 #define LSP_PLUGINS_COMP_DELAY_VERSION_MINOR         0
-#define LSP_PLUGINS_COMP_DELAY_VERSION_MICRO         15
+#define LSP_PLUGINS_COMP_DELAY_VERSION_MICRO         16
 
 #define LSP_PLUGINS_COMP_DELAY_VERSION  \
     LSP_MODULE_VERSION( \
@@ -68,6 +68,7 @@ namespace lsp
             CONTROL("time", "Time", U_MSEC, comp_delay::TIME),
             DRY_GAIN(0.0f),
             WET_GAIN(1.0f),
+            SWITCH("phase", "Phase Invert", 0.0f),
 
             OUT_GAIN,
 
@@ -101,6 +102,8 @@ namespace lsp
             CONTROL("time", "Time", U_MSEC, comp_delay::TIME),
             DRY_GAIN(0.0f),
             WET_GAIN(1.0f),
+            SWITCH("phase_l", "Phase Invert Left", 0.0f),
+            SWITCH("phase_r", "Phase Invert Right", 0.0f),
 
             OUT_GAIN,
 
@@ -125,36 +128,38 @@ namespace lsp
             // Input controls
             BYPASS,
 
-            COMBO("mode_l", "Mode L", 0, comp_delay_modes),
-            SWITCH("ramp_l", "Ramping L", 0.0f),
-            INT_CONTROL("samp_l", "Samples L", U_SAMPLES, comp_delay::SAMPLES),
-            INT_CONTROL("m_l", "Meters L", U_M, comp_delay::METERS),
-            CONTROL("cm_l", "Centimeters L", U_CM, comp_delay::CENTIMETERS),
-            CONTROL("t_l", "Temperature L", U_DEG_CEL, comp_delay::TEMPERATURE),
-            CONTROL("time_l", "Time L", U_MSEC, comp_delay::TIME),
+            COMBO("mode_l", "Mode Left", 0, comp_delay_modes),
+            SWITCH("ramp_l", "Ramping Left", 0.0f),
+            INT_CONTROL("samp_l", "Samples Left", U_SAMPLES, comp_delay::SAMPLES),
+            INT_CONTROL("m_l", "Meters Left", U_M, comp_delay::METERS),
+            CONTROL("cm_l", "Centimeters Left", U_CM, comp_delay::CENTIMETERS),
+            CONTROL("t_l", "Temperature Left", U_DEG_CEL, comp_delay::TEMPERATURE),
+            CONTROL("time_l", "Time Left", U_MSEC, comp_delay::TIME),
             DRY_GAIN_L(0.0f),
             WET_GAIN_L(1.0f),
+            SWITCH("phase_l", "Phase Invert Left", 0.0f),
 
-            COMBO("mode_r", "Mode R", 0, comp_delay_modes),
-            SWITCH("ramp_r", "Ramping R", 0.0f),
-            INT_CONTROL("samp_r", "Samples R", U_SAMPLES, comp_delay::SAMPLES),
-            INT_CONTROL("m_r", "Meters R", U_M, comp_delay::METERS),
-            CONTROL("cm_r", "Centimeters R", U_CM, comp_delay::CENTIMETERS),
-            CONTROL("t_r", "Temperature R", U_DEG_CEL, comp_delay::TEMPERATURE),
-            CONTROL("time_r", "Time R", U_MSEC, comp_delay::TIME),
+            COMBO("mode_r", "Mode Right", 0, comp_delay_modes),
+            SWITCH("ramp_r", "Ramping Right", 0.0f),
+            INT_CONTROL("samp_r", "Samples Right", U_SAMPLES, comp_delay::SAMPLES),
+            INT_CONTROL("m_r", "Meters Right", U_M, comp_delay::METERS),
+            CONTROL("cm_r", "Centimeters Right", U_CM, comp_delay::CENTIMETERS),
+            CONTROL("t_r", "Temperature Right", U_DEG_CEL, comp_delay::TEMPERATURE),
+            CONTROL("time_r", "Time Right", U_MSEC, comp_delay::TIME),
             DRY_GAIN_R(0.0f),
             WET_GAIN_R(1.0f),
+            SWITCH("phase_r", "Phase Invert Right", 0.0f),
 
             OUT_GAIN,
 
             // Output controls
-            METER_MINMAX("d_t_l", "Delay time L", U_HZ, 0.0f, comp_delay::DELAY_OUT_MAX_TIME),
-            METER_MINMAX("d_s_l", "Delay samples L", U_SAMPLES, 0.0f, comp_delay::DELAY_OUT_MAX_SAMPLES),
-            METER_MINMAX("d_d_l", "Delay distance L", U_CM, 0.0f, comp_delay::DELAY_OUT_MAX_DISTANCE),
+            METER_MINMAX("d_t_l", "Delay time Left", U_HZ, 0.0f, comp_delay::DELAY_OUT_MAX_TIME),
+            METER_MINMAX("d_s_l", "Delay samples Left", U_SAMPLES, 0.0f, comp_delay::DELAY_OUT_MAX_SAMPLES),
+            METER_MINMAX("d_d_l", "Delay distance Left", U_CM, 0.0f, comp_delay::DELAY_OUT_MAX_DISTANCE),
 
-            METER_MINMAX("d_t_r", "Delay time R", U_HZ, 0.0f, comp_delay::DELAY_OUT_MAX_TIME),
-            METER_MINMAX("d_s_r", "Delay samples R", U_SAMPLES, 0.0f, comp_delay::DELAY_OUT_MAX_SAMPLES),
-            METER_MINMAX("d_d_r", "Delay distance R", U_CM, 0.0f, comp_delay::DELAY_OUT_MAX_DISTANCE),
+            METER_MINMAX("d_t_r", "Delay time Right", U_HZ, 0.0f, comp_delay::DELAY_OUT_MAX_TIME),
+            METER_MINMAX("d_s_r", "Delay samples Right", U_SAMPLES, 0.0f, comp_delay::DELAY_OUT_MAX_SAMPLES),
+            METER_MINMAX("d_d_r", "Delay distance Right", U_CM, 0.0f, comp_delay::DELAY_OUT_MAX_DISTANCE),
 
             PORTS_END
         };
