@@ -20,6 +20,7 @@
  */
 
 #include <lsp-plug.in/common/alloc.h>
+#include <lsp-plug.in/common/bits.h>
 #include <lsp-plug.in/common/debug.h>
 #include <lsp-plug.in/dsp/dsp.h>
 #include <lsp-plug.in/dsp-units/units.h>
@@ -31,11 +32,10 @@
 #include <private/plugins/gott_compressor.h>
 
 /* The size of temporary buffer for audio processing */
-#define GOTT_BUFFER_SIZE        0x1000U
+#define GOTT_BUFFER_SIZE        0x400U
 
 namespace lsp
 {
-    [[maybe_unused]]
     static plug::IPort *TRACE_PORT(plug::IPort *p)
     {
         lsp_trace("  port id=%s", (p)->metadata()->id);
@@ -114,10 +114,11 @@ namespace lsp
             }
 
             bProt               = true;
-            bModern             = true;
+            enXOver             = XOVER_MODERN;
             bEnvUpdate          = true;
             nBands              = meta::gott_compressor::BANDS_MAX;
             bExtSidechain       = false;
+            bStereoSplit        = false;
             fInGain             = GAIN_AMP_0_DB;
             fDryGain            = GAIN_AMP_M_INF_DB;
             fWetGain            = GAIN_AMP_0_DB;
@@ -133,6 +134,9 @@ namespace lsp
             vAnalyze[2]         = NULL;
             vAnalyze[3]         = NULL;
             vBuffer             = NULL;
+            vProtBuffer         = NULL;
+            vSCIn[0]            = NULL;
+            vSCIn[1]            = NULL;
             vSC[0]              = NULL;
             vSC[1]              = NULL;
             vEnv                = NULL;
@@ -153,6 +157,7 @@ namespace lsp
             pWetGain            = NULL;
             pScMode             = NULL;
             pScSource           = NULL;
+            pScSpSource         = NULL;
             pScPreamp           = NULL;
             pScReact            = NULL;
             pLookahead          = NULL;
@@ -165,6 +170,7 @@ namespace lsp
             pSplits[2]          = NULL;
             pExtraBand          = NULL;
             pExtSidechain       = NULL;
+            pStereoSplit        = NULL;
 
             pData               = NULL;
         }
@@ -197,6 +203,10 @@ namespace lsp
             if (sFilters.init(meta::gott_compressor::BANDS_MAX * channels) != STATUS_OK)
                 return;
 
+            // Initialize surge protection module
+            if (!sProtSC.init(channels, meta::gott_compressor::SC_REACTIVITY_MAX))
+                return;
+
             // Compute amount of memory
             size_t szof_channels    = align_size(sizeof(channel_t) * channels, OPTIMAL_ALIGN);
             size_t szof_buffer      = align_size(sizeof(float) * GOTT_BUFFER_SIZE, OPTIMAL_ALIGN);
@@ -207,6 +217,7 @@ namespace lsp
             size_t to_alloc         =
                 szof_channels +
                 szof_buffer +       // vBuffer
+                szof_buffer +       // vProtBuffer
                 szof_buffer*2 +     // vSC[2]
                 szof_buffer +       // vEnv
                 szof_freq*2 +       // vTr
@@ -224,6 +235,7 @@ namespace lsp
                     szof_freq*2 +   // vTmpFilterBuffer
                     szof_freq +     // vFilterBuffer
                     (
+                        szof_buffer +   // vBuffer
                         szof_buffer +   // vVCA
                         szof_freq*2 +   // vFilterBuffer
                         szof_curve      // vCurveBuffer
@@ -240,6 +252,8 @@ namespace lsp
 
             vBuffer                 = reinterpret_cast<float *>(ptr);
             ptr                    += szof_buffer;
+            vProtBuffer             = reinterpret_cast<float *>(ptr);
+            ptr                    += szof_buffer;
             vSC[0]                  = reinterpret_cast<float *>(ptr);
             ptr                    += szof_buffer;
             vSC[1]                  = reinterpret_cast<float *>(ptr);
@@ -275,6 +289,11 @@ namespace lsp
                 c->sDryEq.construct();
                 c->sDryEq.init(meta::gott_compressor::BANDS_MAX - 1, 0);
                 c->sDryEq.set_mode(dspu::EQM_IIR);
+                c->sFFTXOver.construct();
+                c->sDryDelay.construct();
+                c->sAnDelay.construct();
+                c->sScDelay.construct();
+                c->sXOverDelay.construct();
 
                 c->sDelay.construct();
 
@@ -287,7 +306,6 @@ namespace lsp
                     b->sEQ[0].construct();
                     b->sEQ[1].construct();
                     b->sProc.construct();
-                    b->sProt.construct();
                     b->sPassFilter.construct();
                     b->sRejFilter.construct();
                     b->sAllFilter.construct();
@@ -318,6 +336,8 @@ namespace lsp
                     }
 
                     // Initialize oteher fields
+                    b->vBuffer          = reinterpret_cast<float *>(ptr);
+                    ptr                += szof_buffer;
                     b->vVCA             = reinterpret_cast<float *>(ptr);
                     ptr                += szof_buffer;
                     b->vCurveBuffer     = reinterpret_cast<float *>(ptr);
@@ -379,6 +399,8 @@ namespace lsp
                 c->vFilterBuffer        = reinterpret_cast<float *>(ptr);
                 ptr                    += szof_freq;
 
+                vSCIn[i]                = c->vScBuffer;
+
                 c->nAnInChannel         = an_cid++;
                 c->nAnOutChannel        = an_cid++;
                 vAnalyze[c->nAnInChannel]   = c->vInAnalyze;
@@ -439,6 +461,11 @@ namespace lsp
             pExtraBand              = TRACE_PORT(ports[port_id++]);
             if (bSidechain)
                 pExtSidechain           = TRACE_PORT(ports[port_id++]);
+            if (nMode == GOTT_STEREO)
+            {
+                pStereoSplit            = TRACE_PORT(ports[port_id++]);
+                pScSpSource             = TRACE_PORT(ports[port_id++]);
+            }
             if ((nMode == GOTT_LR) || (nMode == GOTT_MS))
                 TRACE_PORT(ports[port_id++]); // Skip channel selector
 
@@ -470,9 +497,6 @@ namespace lsp
 
                         b->pCurveMesh           = sb->pCurveMesh;
                         b->pFreqMesh            = sb->pFreqMesh;
-                        b->pEnvLvl              = sb->pEnvLvl;
-                        b->pCurveLvl            = sb->pCurveLvl;
-                        b->pMeterGain           = sb->pMeterGain;
                     }
                 }
                 else
@@ -497,13 +521,24 @@ namespace lsp
 
                         b->pCurveMesh           = TRACE_PORT(ports[port_id++]);
                         b->pFreqMesh            = TRACE_PORT(ports[port_id++]);
-                        b->pEnvLvl              = TRACE_PORT(ports[port_id++]);
-                        b->pCurveLvl            = TRACE_PORT(ports[port_id++]);
-                        b->pMeterGain           = TRACE_PORT(ports[port_id++]);
                     }
                 }
             }
 
+            lsp_trace("Binding band meter ports");
+            for (size_t i=0; i<channels; ++i)
+            {
+                channel_t *c            = &vChannels[i];
+                for (size_t j=0; j<meta::gott_compressor::BANDS_MAX; ++j)
+                {
+                    band_t *b               = &c->vBands[j];
+
+                    b->pEnvLvl              = TRACE_PORT(ports[port_id++]);
+                    b->pCurveLvl            = TRACE_PORT(ports[port_id++]);
+                    b->pMeterGain           = TRACE_PORT(ports[port_id++]);
+                }
+            }
+
             lsp_trace("Binding channel meter ports");
             for (size_t i=0; i<channels; ++i)
             {
@@ -516,13 +551,12 @@ namespace lsp
                 c->pOutLvl              = TRACE_PORT(ports[port_id++]);
             }
 
-            if ((nMode == GOTT_LR) || (nMode == GOTT_MS))
+            lsp_trace("Binding aplification curve ports");
+            for (size_t i=0; i<channels; ++i)
             {
-                vChannels[0].pAmpGraph  = TRACE_PORT(ports[port_id++]);
-                vChannels[1].pAmpGraph  = TRACE_PORT(ports[port_id++]);
+                channel_t *c            = &vChannels[i];
+                c->pAmpGraph            = TRACE_PORT(ports[port_id++]);
             }
-            else
-                vChannels[0].pAmpGraph  = TRACE_PORT(ports[port_id++]);
 
             // Initialize curve (logarithmic) in range of -72 .. +24 db
             float delta = (meta::gott_compressor::CURVE_DB_MAX - meta::gott_compressor::CURVE_DB_MIN) / (meta::gott_compressor::CURVE_MESH_SIZE-1);
@@ -540,6 +574,10 @@ namespace lsp
             // Destroy dynamic filters
             sFilters.destroy();
 
+            // Destroy surge protection
+            sProtSC.destroy();
+            sProt.destroy();
+
             // Destroy channels
             if (vChannels != NULL)
             {
@@ -553,15 +591,20 @@ namespace lsp
                     c->sEnvBoost[0].destroy();
                     c->sEnvBoost[1].destroy();
                     c->sDryEq.destroy();
+                    c->sFFTXOver.destroy();
                     c->sDelay.destroy();
+                    c->sDryDelay.destroy();
+                    c->sAnDelay.destroy();
+                    c->sScDelay.destroy();
+                    c->sXOverDelay.destroy();
 
                     for (size_t j=0; j<meta::gott_compressor::BANDS_MAX; ++j)
                     {
                         band_t *b               = &c->vBands[j];
 
+                        b->sSC.destroy();
                         b->sEQ[0].destroy();
                         b->sEQ[1].destroy();
-                        b->sSC.destroy();
 
                         b->sPassFilter.destroy();
                         b->sRejFilter.destroy();
@@ -604,15 +647,25 @@ namespace lsp
             }
         }
 
+        size_t gott_compressor::select_fft_rank(size_t sample_rate)
+        {
+            const size_t k = (sample_rate + meta::gott_compressor::FFT_XOVER_FREQ_MIN/2) / meta::gott_compressor::FFT_XOVER_FREQ_MIN;
+            const size_t n = int_log2(k);
+            return meta::gott_compressor::FFT_XOVER_RANK_MIN + n;
+        }
+
         void gott_compressor::update_sample_rate(long sr)
         {
             // Determine number of channels
             size_t channels     = (nMode == GOTT_MONO) ? 1 : 2;
-            size_t max_delay    = dspu::millis_to_samples(sr, meta::gott_compressor::LOOKAHEAD_MAX);
+            size_t fft_rank     = select_fft_rank(sr);
+            size_t bins         = 1 << fft_rank;
+            size_t max_delay    = bins + dspu::millis_to_samples(sr, meta::gott_compressor::LOOKAHEAD_MAX);
 
             // Update analyzer's sample rate
             sAnalyzer.set_sample_rate(sr);
             sFilters.set_sample_rate(sr);
+            sProtSC.set_sample_rate(sr);
             bEnvUpdate          = true;
 
             // Update channels
@@ -621,8 +674,23 @@ namespace lsp
                 channel_t *c = &vChannels[i];
 
                 c->sBypass.init(sr);
-                c->sDelay.init(max_delay);
                 c->sDryEq.set_sample_rate(sr);
+                c->sDelay.init(max_delay);
+                c->sDryDelay.init(max_delay);
+                c->sAnDelay.init(bins);
+                c->sScDelay.init(bins);
+                c->sXOverDelay.init(max_delay);
+
+                // Need to re-initialize FFT crossover?
+                if (fft_rank != c->sFFTXOver.rank())
+                {
+                    c->sFFTXOver.init(fft_rank, meta::gott_compressor::BANDS_MAX);
+                    for (size_t j=0; j<meta::gott_compressor::BANDS_MAX; ++j)
+                        c->sFFTXOver.set_handler(j, process_band, this, c);
+                    c->sFFTXOver.set_rank(fft_rank);
+                    c->sFFTXOver.set_phase(float(i) / float(channels));
+                }
+                c->sFFTXOver.set_sample_rate(sr);
 
                 // Update bands
                 for (size_t j=0; j<meta::gott_compressor::BANDS_MAX; ++j)
@@ -646,6 +714,52 @@ namespace lsp
             }
         }
 
+        dspu::sidechain_source_t gott_compressor::decode_sidechain_source(int source, bool split, size_t channel)
+        {
+            if (!split)
+            {
+                switch (source)
+                {
+                    case 0: return dspu::SCS_MIDDLE;
+                    case 1: return dspu::SCS_SIDE;
+                    case 2: return dspu::SCS_LEFT;
+                    case 3: return dspu::SCS_RIGHT;
+                    case 4: return dspu::SCS_AMIN;
+                    case 5: return dspu::SCS_AMAX;
+                    default: break;
+                }
+            }
+
+            if (channel == 0)
+            {
+                switch (source)
+                {
+                    case 0: return dspu::SCS_LEFT;
+                    case 1: return dspu::SCS_RIGHT;
+                    case 2: return dspu::SCS_MIDDLE;
+                    case 3: return dspu::SCS_SIDE;
+                    case 4: return dspu::SCS_AMIN;
+                    case 5: return dspu::SCS_AMAX;
+                    default: break;
+                }
+            }
+            else
+            {
+                switch (source)
+                {
+                    case 0: return dspu::SCS_RIGHT;
+                    case 1: return dspu::SCS_LEFT;
+                    case 2: return dspu::SCS_SIDE;
+                    case 3: return dspu::SCS_MIDDLE;
+                    case 4: return dspu::SCS_AMIN;
+                    case 5: return dspu::SCS_AMAX;
+                    default: break;
+                }
+            }
+
+            return dspu::SCS_MIDDLE;
+        }
+
         void gott_compressor::update_settings()
         {
             dspu::filter_params_t fp;
@@ -659,6 +773,16 @@ namespace lsp
             float sc_preamp     = pScPreamp->value();
             size_t lookahead    = dspu::millis_to_samples(fSampleRate, pLookahead->value());
 
+            // Determine work mode: classic, modern or linear phase
+            xover_mode_t xover  = xover_mode_t(pMode->value());
+            if (xover != enXOver)
+            {
+                enXOver             = xover;
+                rebuild_filters     = true;
+                for (size_t i=0; i<channels; ++i)
+                    vChannels[i].sXOverDelay.clear();
+            }
+
             // Check band and split configuration
             if (nBands != num_bands)
             {
@@ -674,6 +798,7 @@ namespace lsp
                     vSplits[i]          = freq;
                 }
             }
+            bStereoSplit        = (pStereoSplit != NULL) ? pStereoSplit->value() >= 0.5f : false;
 
             // Store gain
             float out_gain      = pOutGain->value();
@@ -682,6 +807,11 @@ namespace lsp
             fWetGain            = out_gain * pWetGain->value();
             fZoom               = pZoom->value();
             bExtSidechain       = (pExtSidechain != NULL) ? pExtSidechain->value() > 0.5f : false;
+            plug::IPort *sc     = (bStereoSplit) ? pScSpSource : pScSource;
+            size_t sc_src       = (sc != NULL) ? sc->value() : dspu::SCS_MIDDLE;
+
+            float max_attack    = meta::gott_compressor::ATTACK_TIME_MIN;
+            float sc_react      = pScReact->value();
 
             for (size_t i=0; i<channels; ++i)
             {
@@ -695,8 +825,8 @@ namespace lsp
                 c->sBypass.set_bypass(pBypass->value());
 
                 // Update analyzer settings
-                c->bInFft           = c->pFftInSw->value() >= 0.5f;
-                c->bOutFft          = c->pFftOutSw->value() >= 0.5f;
+                c->bInFft               = c->pFftInSw->value() >= 0.5f;
+                c->bOutFft              = c->pFftOutSw->value() >= 0.5f;
 
                 sAnalyzer.enable_channel(c->nAnInChannel, c->bInFft);
                 sAnalyzer.enable_channel(c->nAnOutChannel, c->pFftOutSw->value()  >= 0.5f);
@@ -715,13 +845,12 @@ namespace lsp
                     bool enabled            = (j < nBands) && (b->pEnabled->value() >= 0.5f);
                     bool mute               = (b->pMute->value() >= 0.5f);
                     bool solo               = (b->pSolo->value() >= 0.5f);
-                    float sc_react          = pScReact->value();
 
                     // Update sidechain settings
                     b->sSC.set_mode(pScMode->value());
                     b->sSC.set_reactivity(sc_react);
                     b->sSC.set_stereo_mode((nMode == GOTT_MS) ? dspu::SCSM_MIDSIDE : dspu::SCSM_STEREO);
-                    b->sSC.set_source((pScSource != NULL) ? pScSource->value() : dspu::SCS_MIDDLE);
+                    b->sSC.set_source(decode_sidechain_source(sc_src, bStereoSplit, i));
 
                     if (sc_preamp != fScPreamp)
                         b->nSync       |= S_EQ_CURVE;
@@ -737,13 +866,7 @@ namespace lsp
                     float f_min_gain        = lsp_min(b->pMinThresh->value(), f_up_gain * 0.999f);
                     float f_min_value       = f_up_gain - (f_up_gain - f_min_gain) / up_ratio;
 
-                    // Update surge protection
-                    b->sProt.set_on_threshold(f_min_gain);
-                    b->sProt.set_off_threshold(meta::gott_compressor::THRESH_MIN_MIN * GAIN_AMP_M_12_DB);
-                    b->sProt.set_transition_time(dspu::millis_to_samples(fSampleRate, attack + sc_react) * meta::gott_compressor::PROT_ATTACK_MUL);
-                    b->sProt.set_shutdown_time(dspu::millis_to_samples(fSampleRate, meta::gott_compressor::PROT_SHUTDOWN_TIME));
-                    if (prot_on && (!bProt))
-                        b->sProt.reset();
+                    max_attack              = lsp_max(max_attack, attack);
 
                     // Update dynamics processor
                     b->sProc.set_attack_time(0, attack);
@@ -813,11 +936,22 @@ namespace lsp
                     if (bSidechain)
                         c->sEnvBoost[1].update(fSampleRate, &fp);
                 }
-
-                // Update lookahead delay
-                c->sDelay.set_delay(lookahead);
             }
 
+            // Update surge protection sidechain settings
+            sProtSC.set_mode(pScMode->value());
+            sProtSC.set_reactivity(sc_react);
+            sProtSC.set_stereo_mode((nMode == GOTT_MS) ? dspu::SCSM_MIDSIDE : dspu::SCSM_STEREO);
+            sProtSC.set_source(dspu::SCS_AMAX);
+
+            // Update surge protection
+            sProt.set_on_threshold(GAIN_AMP_M_96_DB);
+            sProt.set_off_threshold(meta::gott_compressor::THRESH_MIN_MIN * GAIN_AMP_M_12_DB);
+            sProt.set_transition_time(dspu::millis_to_samples(fSampleRate, max_attack + sc_react) * meta::gott_compressor::PROT_ATTACK_MUL);
+            sProt.set_shutdown_time(dspu::millis_to_samples(fSampleRate, meta::gott_compressor::PROT_SHUTDOWN_TIME));
+            if (prot_on && (!bProt))
+                sProt.reset();
+
             // Commit the envelope state
             fScPreamp       = sc_preamp;
             nEnvBoost       = env_boost;
@@ -861,7 +995,8 @@ namespace lsp
                     // Configure equalizers
                     for (size_t j=0; j<nBands; ++j)
                     {
-                        band_t *b   = &c->vBands[j];
+                        band_t *b           = &c->vBands[j];
+                        size_t band         = b - c->vBands;
 
                         float freq_start    = (j > 0) ? vSplits[j-1] : 0.0f;
                         float freq_end      = (j < (nBands - 1)) ? vSplits[j] : fSampleRate * 0.5f;
@@ -901,7 +1036,7 @@ namespace lsp
                         dsp::pcomplex_mod(b->vFilterBuffer, b->vFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
 
                         // Update filter parameters, depending on operating mode
-                        if (bModern)
+                        if (enXOver == XOVER_MODERN)
                         {
                             // Configure filter for band
                             if (j <= 0)
@@ -932,7 +1067,7 @@ namespace lsp
                             sFilters.set_params(b->nFilterID, &fp);
                             sFilters.set_filter_active(b->nFilterID, j < nBands);
                         }
-                        else
+                        else if (enXOver == XOVER_CLASSIC)
                         {
                             fp.fGain        = 1.0f;
                             fp.nSlope       = 2;
@@ -958,6 +1093,34 @@ namespace lsp
                                 b->sAllFilter.update(fSampleRate, &fp);
                             }
                         }
+                        else // enXOver == XOVER_LINEAR_PHASE
+                        {
+                            if (j > 0)
+                            {
+                                c->sFFTXOver.enable_hpf(band, true);
+                                c->sFFTXOver.set_hpf_frequency(band, freq_start);
+                                c->sFFTXOver.set_hpf_slope(band, -48.0f);
+                            }
+                            else
+                                c->sFFTXOver.disable_hpf(band);
+
+                            if (j < (nBands-1))
+                            {
+                                c->sFFTXOver.enable_lpf(band, true);
+                                c->sFFTXOver.set_lpf_frequency(band, freq_end);
+                                c->sFFTXOver.set_lpf_slope(band, -48.0f);
+                            }
+                            else
+                                c->sFFTXOver.disable_lpf(band);
+                        }
+                    }
+
+                    // Enable/disable bands in FFT crossover
+                    for (size_t j=0; j<meta::gott_compressor::BANDS_MAX; ++j)
+                    {
+                        band_t *b       = &c->vBands[j];
+                        size_t band     = b - c->vBands;
+                        c->sFFTXOver.enable_band(band, j < nBands);
                     }
 
                     // Set-up all-pass filters for the 'dry' chain which can be mixed with the 'wet' chain.
@@ -981,7 +1144,28 @@ namespace lsp
             }
 
             // Report latency
-            set_latency(lookahead);
+            size_t xover_latency = (enXOver == XOVER_LINEAR_PHASE) ? vChannels[0].sFFTXOver.latency() : 0;
+
+            set_latency(lookahead + xover_latency);
+            for (size_t i=0; i<channels; ++i)
+            {
+                channel_t *c    = &vChannels[i];
+
+                c->sDelay.set_delay(lookahead);
+                c->sDryDelay.set_delay(lookahead + xover_latency);
+                c->sAnDelay.set_delay(xover_latency);
+                c->sScDelay.set_delay(xover_latency);
+                c->sXOverDelay.set_delay(lookahead + xover_latency);
+            }
+        }
+
+        void gott_compressor::process_band(void *object, void *subject, size_t band, const float *data, size_t sample, size_t count)
+        {
+            channel_t *c            = static_cast<channel_t *>(subject);
+            band_t *b               = &c->vBands[band];
+
+            // Store data to band's buffer
+            dsp::copy(&b->vBuffer[sample], data, count);
         }
 
         void gott_compressor::process(size_t samples)
@@ -1060,9 +1244,14 @@ namespace lsp
                 {
                     channel_t *c        = &vChannels[i];
                     c->sEnvBoost[0].process(c->vScBuffer, c->vScBuffer, to_process);
-                    dsp::copy(c->vInAnalyze, c->vBuffer, to_process);
+                    c->sAnDelay.process(c->vInAnalyze, c->vBuffer, to_process);
+                    c->sScDelay.process(c->vScBuffer, c->vScBuffer, to_process);
                 }
 
+                // Surge protection
+                sProtSC.process(vProtBuffer, vSCIn, to_process);
+                sProt.process(vProtBuffer, vProtBuffer, to_process);
+
                 // MAIN PLUGIN STUFF
                 for (size_t i=0; i<channels; ++i)
                 {
@@ -1078,7 +1267,7 @@ namespace lsp
                             b->sEQ[1].process(vSC[1], vChannels[1].vScBuffer, to_process);
 
                         // Preprocess VCA signal
-                        b->sSC.process(vBuffer, const_cast<const float **>(vSC), to_process); // Band now contains processed by sidechain signal
+                        b->sSC.process(vBuffer, const_cast<const float **>(vSC), to_process);   // Band now contains processed by sidechain signal
                         dsp::mul_k2(vBuffer, fScPreamp, to_process);
 
                         if (b->bEnabled)
@@ -1097,20 +1286,19 @@ namespace lsp
                             b->fGainLevel   = b->vVCA[to_process-1];
 
                             // Check muting option
-                            dsp::mul_k2(b->vVCA, b->fMakeup, to_process); // Apply makeup gain
                             if (bProt)
-                                b->sProt.process_mul(b->vVCA, vBuffer, to_process);
+                                dsp::fmmul_k3(b->vVCA, vProtBuffer, b->fMakeup, to_process);
+                            else
+                                dsp::mul_k2(b->vVCA, b->fMakeup, to_process); // Apply makeup gain
 
                             // Patch the VCA signal
                             if (b->bMute)
                                 dsp::fill(b->vVCA, GAIN_AMP_M_36_DB, to_process);
-                            else if (bModern)
+                            else if (enXOver == XOVER_MODERN) // 'Modern' mode
                                 dsp::limit1(b->vVCA, GAIN_AMP_M_72_DB * b->fMakeup, GAIN_AMP_P_72_DB * b->fMakeup, to_process);
                         }
                         else
                         {
-                            if (bProt)
-                                b->sProt.process(vBuffer, to_process);
                             dsp::fill(b->vVCA, (b->bMute) ? GAIN_AMP_M_36_DB : GAIN_AMP_0_DB, to_process);
                             b->fGainLevel   = GAIN_AMP_0_DB;
                         }
@@ -1130,7 +1318,7 @@ namespace lsp
                 }
 
                 // Here, we apply VCA to input signal dependent on the input
-                if (bModern) // 'Modern' mode
+                if (enXOver == XOVER_MODERN) // 'Modern' mode
                 {
                     // Apply VCA control
                     for (size_t i=0; i<channels; ++i)
@@ -1146,7 +1334,7 @@ namespace lsp
                         }
                     }
                 }
-                else // 'Classic' mode
+                else if (enXOver == XOVER_CLASSIC) // 'Classic' mode
                 {
                     // Apply VCA control
                     for (size_t i=0; i<channels; ++i)
@@ -1162,11 +1350,35 @@ namespace lsp
                         {
                             band_t *b       = &c->vBands[j];
 
-                            b->sAllFilter.process(c->vBuffer, c->vBuffer, to_process); // Process the signal with all-pass
-                            b->sPassFilter.process(vEnv, vBuffer, to_process); // Filter frequencies from input
-                            dsp::mul2(vEnv, b->vVCA, to_process); // Apply VCA gain
-                            dsp::add2(c->vBuffer, vEnv, to_process); // Add signal to the channel buffer
-                            b->sRejFilter.process(vBuffer, vBuffer, to_process); // Filter frequencies from input
+                            // Process the signal with all-pass
+                            b->sAllFilter.process(c->vBuffer, c->vBuffer, to_process);
+                            // Filter frequencies from input
+                            b->sPassFilter.process(vEnv, vBuffer, to_process);
+                            // Apply VCA gain and add to the channel buffer
+                            dsp::fmadd3(c->vBuffer, vEnv, b->vVCA, to_process);
+                            // Filter frequencies from input
+                            b->sRejFilter.process(vBuffer, vBuffer, to_process);
+                        }
+                    }
+                }
+                else // enXOver == XOVER_LINEAR_PHASE
+                {
+                    // Apply VCA control
+                    for (size_t i=0; i<channels; ++i)
+                    {
+                        channel_t *c        = &vChannels[i];
+
+                        // Apply delay to compensate lookahead feature
+                        c->sDelay.process(c->vBuffer, c->vBuffer, to_process);
+                        // Apply delay to unprocessed signal to compensate lookahead + crossover delay
+                        c->sXOverDelay.process(c->vInBuffer, c->vBuffer, to_process);
+                        c->sFFTXOver.process(c->vBuffer, to_process);
+                        dsp::fill_zero(c->vBuffer, to_process);                 // Clear the channel buffer
+
+                        for (size_t j=0; j<nBands; ++j)
+                        {
+                            band_t *b           = &c->vBands[j];
+                            dsp::fmadd3(c->vBuffer, b->vVCA, b->vBuffer, to_process);
                         }
                     }
                 }
@@ -1195,20 +1407,23 @@ namespace lsp
                     channel_t *c        = &vChannels[i];
 
                     // Apply dry/wet balance
-                    if (bModern)
+                    if (enXOver == XOVER_MODERN)
                         dsp::mix2(c->vBuffer, c->vInBuffer, fWetGain, fDryGain, to_process);
-                    else
+                    else if (enXOver == XOVER_CLASSIC)
                     {
                         c->sDryEq.process(vBuffer, c->vInBuffer, to_process);
                         dsp::mix2(c->vBuffer, vBuffer, fWetGain, fDryGain, to_process);
                     }
+                    else // enXOver == XOVER_LINEAR_PHASE
+                        dsp::mix2(c->vBuffer, c->vInBuffer, fWetGain, fDryGain, to_process);
 
                     // Compute output level
                     float level         = dsp::abs_max(c->vBuffer, to_process);
                     c->pOutLvl->set_value(level);
 
                     // Apply bypass
-                    c->sBypass.process(c->vOut, c->vIn, c->vBuffer, to_process);
+                    c->sDryDelay.process(vBuffer, c->vIn, to_process);
+                    c->sBypass.process(c->vOut, vBuffer, c->vBuffer, to_process);
 
                     // Update pointers
                     c->vIn             += to_process;
@@ -1228,7 +1443,7 @@ namespace lsp
                 channel_t *c        = &vChannels[i];
 
                 // Calculate transfer function for the compressor
-                if (bModern)
+                if (enXOver == XOVER_MODERN)
                 {
                     dsp::pcomplex_fill_ri(c->vTmpFilterBuffer, 1.0f, 0.0f, meta::gott_compressor::FFT_MESH_POINTS);
 
@@ -1239,8 +1454,9 @@ namespace lsp
                         sFilters.freq_chart(b->nFilterID, vTr, vFreqBuffer, b->fGainLevel, meta::gott_compressor::FFT_MESH_POINTS);
                         dsp::pcomplex_mul2(c->vTmpFilterBuffer, vTr, meta::gott_compressor::FFT_MESH_POINTS);
                     }
+                    dsp::pcomplex_mod(c->vFilterBuffer, c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
                 }
-                else
+                else if (enXOver == XOVER_CLASSIC)
                 {
                     dsp::pcomplex_fill_ri(vTr, 1.0f, 0.0f, meta::gott_compressor::FFT_MESH_POINTS);
                     dsp::fill_zero(c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS*2);
@@ -1263,8 +1479,21 @@ namespace lsp
                         b->sRejFilter.freq_chart(vRFc, vFreqBuffer, meta::gott_compressor::FFT_MESH_POINTS);
                         dsp::pcomplex_mul2(vTr, vRFc, meta::gott_compressor::FFT_MESH_POINTS);
                     }
+                    dsp::pcomplex_mod(c->vFilterBuffer, c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
+                }
+                else // enXOver == XOVER_LINEAR_PHASE
+                {
+                    dsp::fill_zero(c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
+                    // Calculate transfer function
+                    for (size_t j=0; j<nBands; ++j)
+                    {
+                        band_t *b           = &c->vBands[j];
+                        size_t band         = b - c->vBands;
+                        c->sFFTXOver.freq_chart(band, vPFc, vFreqBuffer, meta::gott_compressor::FFT_MESH_POINTS);
+                        dsp::fmadd_k3(c->vTmpFilterBuffer, vPFc, b->fGainLevel, meta::gott_compressor::FFT_MESH_POINTS);
+                    }
+                    dsp::copy(c->vFilterBuffer, c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
                 }
-                dsp::pcomplex_mod(c->vFilterBuffer, c->vTmpFilterBuffer, meta::gott_compressor::FFT_MESH_POINTS);
 
                 // Output Channel curve
                 mesh            = (c->pAmpGraph != NULL) ? c->pAmpGraph->buffer<plug::mesh_t>() : NULL;
@@ -1433,15 +1662,19 @@ namespace lsp
             b->v[3][0]          = 1.0f;
             b->v[3][width+1]    = 1.0f;
 
-            size_t channels = ((nMode == GOTT_MONO) || (nMode == GOTT_STEREO)) ? 1 : 2;
-            static uint32_t c_colors[] = {
-                    CV_MIDDLE_CHANNEL, CV_MIDDLE_CHANNEL,
-                    CV_MIDDLE_CHANNEL, CV_MIDDLE_CHANNEL,
-                    CV_LEFT_CHANNEL, CV_RIGHT_CHANNEL,
-                    CV_MIDDLE_CHANNEL, CV_SIDE_CHANNEL
-                   };
+            static const uint32_t c_colors[] = {
+                CV_MIDDLE_CHANNEL,
+                CV_LEFT_CHANNEL, CV_RIGHT_CHANNEL,
+                CV_MIDDLE_CHANNEL, CV_SIDE_CHANNEL
+            };
+
+            size_t channels     = ((nMode == GOTT_MONO) || ((nMode == GOTT_STEREO) && (!bStereoSplit))) ? 1 : 2;
+            const uint32_t *vc  = (channels == 1) ? &c_colors[0] :
+                                  (nMode == GOTT_MS) ? &c_colors[3] :
+                                  &c_colors[1];
 
             bool aa = cv->set_anti_aliasing(true);
+            lsp_finally { cv->set_anti_aliasing(aa); };
             cv->set_line_width(2);
 
             for (size_t i=0; i<channels; ++i)
@@ -1461,29 +1694,33 @@ namespace lsp
                 dsp::axis_apply_log1(b->v[2], b->v[3], zy, dy, width+2);
 
                 // Draw mesh
-                uint32_t color = (bypassing || !(active())) ? CV_SILVER : c_colors[nMode*2 + i];
+                uint32_t color = (bypassing || !(active())) ? CV_SILVER : vc[i];
                 Color stroke(color), fill(color, 0.5f);
                 cv->draw_poly(b->v[1], b->v[2], width+2, stroke, fill);
             }
-            cv->set_anti_aliasing(aa);
 
             return true;
         }
 
         void gott_compressor::dump(dspu::IStateDumper *v) const
         {
+            plug::Module::dump(v);
+
             size_t channels     = (nMode == GOTT_MONO) ? 1 : 2;
 
             v->write_object("sAnalyzer", &sAnalyzer);
             v->write_object("sFilters", &sFilters);
+            v->write_object("sProtSC", &sProtSC);
+            v->write_object("sProt", &sProt);
 
             v->write("nMode", nMode);
             v->write("bSidechain", bSidechain);
             v->write("bProt", bProt);
-            v->write("bModern", bModern);
+            v->write("enXOver", enXOver);
             v->write("bEnvUpdate", bEnvUpdate);
             v->write("nBands", nBands);
             v->write("bExtSidechain", bExtSidechain);
+            v->write("bStereoSplit", bStereoSplit);
             v->write("fInGain", fInGain);
             v->write("fDryGain", fDryGain);
             v->write("fWetGain", fWetGain);
@@ -1502,7 +1739,12 @@ namespace lsp
                     v->write_object("sBypass", &c->sBypass);
                     v->write_object_array("sEnvBoost", c->sEnvBoost, 2);
                     v->write_object("sDryEq", &c->sBypass);
+                    v->write_object("sFFTXOver", &c->sFFTXOver);
                     v->write_object("sDelay", &c->sBypass);
+                    v->write_object("sDryDelay", &c->sDryDelay);
+                    v->write_object("sAnDelay", &c->sAnDelay);
+                    v->write_object("sScDelay", &c->sScDelay);
+                    v->write_object("sXOverDelay", &c->sXOverDelay);
 
                     {
                         v->begin_array("vBands", c->vBands, meta::gott_compressor::BANDS_MAX);
@@ -1518,7 +1760,6 @@ namespace lsp
                             v->write_object("sSC", &b->sSC);
                             v->write_object_array("sEQ", b->sEQ, 2);
                             v->write_object("sProc", &b->sProc);
-                            v->write_object("sProt", &b->sProt);
                             v->write_object("sPassFilter", &b->sPassFilter);
                             v->write_object("sRejFilter", &b->sRejFilter);
                             v->write_object("sAllFilter", &b->sAllFilter);
@@ -1612,6 +1853,7 @@ namespace lsp
             v->write("pWetGain", pWetGain);
             v->write("pScMode", pScMode);
             v->write("pScSource", pScSource);
+            v->write("pScSpSource", pScSpSource);
             v->write("pScPreamp", pScPreamp);
             v->write("pScReact", pScReact);
             v->write("pLookahead", pLookahead);
@@ -1622,6 +1864,7 @@ namespace lsp
             v->writev("pSplits", pSplits, meta::gott_compressor::BANDS_MAX - 1);
             v->write("pExtraBand", pExtraBand);
             v->write("pExtSidechain", pExtSidechain);
+            v->write("pStereoSplit", pStereoSplit);
 
             v->write("pData", pData);
         }
