@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2021 Linux Studio Plugins Project <https://lsp-plug.in/>
- *           (C) 2021 Vladimir Sadovnikov <sadko4u@gmail.com>
+ * Copyright (C) 2023 Linux Studio Plugins Project <https://lsp-plug.in/>
+ *           (C) 2023 Vladimir Sadovnikov <sadko4u@gmail.com>
  *
  * This file is part of lsp-plugins-mb-expander
  * Created on: 3 авг. 2021 г.
@@ -26,7 +26,7 @@
 
 #define LSP_PLUGINS_MB_EXPANDER_VERSION_MAJOR       1
 #define LSP_PLUGINS_MB_EXPANDER_VERSION_MINOR       0
-#define LSP_PLUGINS_MB_EXPANDER_VERSION_MICRO       11
+#define LSP_PLUGINS_MB_EXPANDER_VERSION_MICRO       12
 
 #define LSP_PLUGINS_MB_EXPANDER_VERSION  \
     LSP_MODULE_VERSION( \
@@ -65,27 +65,39 @@ namespace lsp
             { NULL, NULL }
         };
 
+        static const port_item_t mb_exp_sc_split_source[] =
+        {
+            { "Left/Right",     "sidechain.left_right"      },
+            { "Right/Left",     "sidechain.right_left"      },
+            { "Mid/Side",       "sidechain.mid_side"        },
+            { "Side/Mid",       "sidechain.side_mid"        },
+            { "Min",            "sidechain.min"             },
+            { "Max",            "sidechain.max"             },
+            { NULL, NULL }
+        };
+
         static const port_item_t mb_exp_sc_boost[] =
         {
-            { "None",       "sidechain.boost.none" },
-            { "Pink BT",    "sidechain.boost.pink_bt" },
-            { "Pink MT",    "sidechain.boost.pink_mt" },
-            { "Brown BT",   "sidechain.boost.brown_bt" },
-            { "Brown MT",   "sidechain.boost.brown_mt" },
+            { "None",           "sidechain.boost.none"      },
+            { "Pink BT",        "sidechain.boost.pink_bt"   },
+            { "Pink MT",        "sidechain.boost.pink_mt"   },
+            { "Brown BT",       "sidechain.boost.brown_bt"  },
+            { "Brown MT",       "sidechain.boost.brown_mt"  },
             { NULL, NULL }
         };
 
         static const port_item_t mb_global_mb_exp_modes[] =
         {
-            { "Classic",    "multiband.classic" },
-            { "Modern",     "multiband.modern" },
+            { "Classic",        "multiband.classic"         },
+            { "Modern",         "multiband.modern"          },
+            { "Linear Phase",   "multiband.linear_phase"    },
             { NULL, NULL }
         };
 
         static const port_item_t mb_exp_modes[] =
         {
-            { "Down",       "mb_expand.down_ward" },
-            { "Up",         "mb_expand.up_ward" },
+            { "Down",       "mb_expand.down_ward"           },
+            { "Up",         "mb_expand.up_ward"             },
             { NULL, NULL }
         };
 
@@ -150,16 +162,6 @@ namespace lsp
             SWITCH("flt" id, "Band filter curves" label, 1.0f), \
             MESH("ag" id, "Expander amplitude graph " label, 2, mb_expander_metadata::FFT_MESH_POINTS)
 
-        #define MB_FFT_METERS(id, label) \
-            SWITCH("ife" id, "Input FFT graph enable" label, 1.0f), \
-            SWITCH("ofe" id, "Output FFT graph enable" label, 1.0f), \
-            MESH("ifg" id, "Input FFT graph" label, 2, mb_expander_metadata::FFT_MESH_POINTS), \
-            MESH("ofg" id, "Output FFT graph" label, 2, mb_expander_metadata::FFT_MESH_POINTS)
-
-        #define MB_CHANNEL_METERS(id, label) \
-            METER_GAIN("ilm" id, "Input level meter" label, GAIN_AMP_P_24_DB), \
-            METER_GAIN("olm" id, "Output level meter" label, GAIN_AMP_P_24_DB)
-
         #define MB_SPLIT(id, label, enable, freq) \
             SWITCH("cbe" id, "Expander band enable" label, enable), \
             LOG_CONTROL_DFL("sf" id, "Split frequency" label, U_HZ, mb_expander_metadata::FREQ, freq)
@@ -189,13 +191,20 @@ namespace lsp
             HUE_CTL("hue" id, "Hue " label, (float(x) / float(total))), \
             METER("fre" id, "Frequency range end" label, U_HZ,  mb_expander_metadata::OUT_FREQ), \
             MESH("ccg" id, "Expander curve graph" label, 2, mb_expander_metadata::CURVE_MESH_SIZE), \
-            METER_OUT_GAIN("rl" id, "Release level" label, 20.0f), \
+            METER_OUT_GAIN("rl" id, "Release level" label, 20.0f) \
+
+        #define MB_BAND_METERS(id, label) \
             METER_OUT_GAIN("elm" id, "Envelope level meter" label, GAIN_AMP_P_36_DB), \
             METER_OUT_GAIN("clm" id, "Curve level meter" label, GAIN_AMP_P_36_DB), \
             METER_OUT_GAIN("rlm" id, "Reduction level meter" label, GAIN_AMP_P_24_DB)
 
         #define MB_STEREO_BAND(id, label, x, total, fe, fs) \
-            COMBO("scs" id, "Sidechain source" label, dspu::SCS_MIDDLE, mb_exp_sc_source), \
+            COMBO("scs" id, "Sidechain source" label, 0, mb_exp_sc_source), \
+            COMBO("sscs" id, "Split sidechain source" label, 0, mb_exp_sc_split_source), \
+            MB_MONO_BAND(id, label, x, total, fe, fs)
+
+        #define MB_SPLIT_BAND(id, label, x, total, fe, fs) \
+            COMBO("scs" id, "Sidechain source" label, 0, mb_exp_sc_source), \
             MB_MONO_BAND(id, label, x, total, fe, fs)
 
         #define MB_SC_MONO_BAND(id, label, x, total, fe, fs) \
@@ -206,6 +215,27 @@ namespace lsp
             SWITCH("sce" id, "External sidechain enable" label, 0.0f), \
             MB_STEREO_BAND(id, label, x, total, fe, fs)
 
+        #define MB_SC_SPLIT_BAND(id, label, x, total, fe, fs) \
+            SWITCH("sce" id, "External sidechain enable" label, 0.0f), \
+            MB_SPLIT_BAND(id, label, x, total, fe, fs)
+
+        #define MB_STEREO_CHANNEL \
+            SWITCH("flt", "Band filter curves", 1.0f), \
+            MESH("ag_l", "Expander amplitude graph Left", 2, mb_expander_metadata::FFT_MESH_POINTS), \
+            MESH("ag_r", "Expander amplitude graph Right", 2, mb_expander_metadata::FFT_MESH_POINTS), \
+            SWITCH("ssplit", "Stereo split", 0.0f)
+
+        #define MB_FFT_METERS(id, label) \
+            SWITCH("ife" id, "Input FFT graph enable" label, 1.0f), \
+            SWITCH("ofe" id, "Output FFT graph enable" label, 1.0f), \
+            MESH("ifg" id, "Input FFT graph" label, 2, mb_expander_metadata::FFT_MESH_POINTS + 2), \
+            MESH("ofg" id, "Output FFT graph" label, 2, mb_expander_metadata::FFT_MESH_POINTS)
+
+        #define MB_CHANNEL_METERS(id, label) \
+            METER_GAIN("ilm" id, "Input level meter" label, GAIN_AMP_P_24_DB), \
+            METER_GAIN("olm" id, "Output level meter" label, GAIN_AMP_P_24_DB)
+
+
         static const port_t mb_expander_mono_ports[] =
         {
             PORTS_MONO_PLUGIN,
@@ -231,6 +261,15 @@ namespace lsp
             MB_MONO_BAND("_6", " 6", 6, 8, 3984.0f, 10000.0f),
             MB_MONO_BAND("_7", " 7", 7, 8, 10000.0f, 20000.0f),
 
+            MB_BAND_METERS("_0", " 0"),
+            MB_BAND_METERS("_1", " 1"),
+            MB_BAND_METERS("_2", " 2"),
+            MB_BAND_METERS("_3", " 3"),
+            MB_BAND_METERS("_4", " 4"),
+            MB_BAND_METERS("_5", " 5"),
+            MB_BAND_METERS("_6", " 6"),
+            MB_BAND_METERS("_7", " 7"),
+
             PORTS_END
         };
 
@@ -238,7 +277,7 @@ namespace lsp
         {
             PORTS_STEREO_PLUGIN,
             MB_COMMON(exp_sc_bands),
-            MB_CHANNEL("", ""),
+            MB_STEREO_CHANNEL,
             MB_FFT_METERS("_l", " Left"),
             MB_CHANNEL_METERS("_l", " Left"),
             MB_FFT_METERS("_r", " Right"),
@@ -261,6 +300,24 @@ namespace lsp
             MB_STEREO_BAND("_6", " 6", 6, 8, 3984.0f, 10000.0f),
             MB_STEREO_BAND("_7", " 7", 7, 8, 10000.0f, 20000.0f),
 
+            MB_BAND_METERS("_0l", " 0 Left"),
+            MB_BAND_METERS("_1l", " 1 Left"),
+            MB_BAND_METERS("_2l", " 2 Left"),
+            MB_BAND_METERS("_3l", " 3 Left"),
+            MB_BAND_METERS("_4l", " 4 Left"),
+            MB_BAND_METERS("_5l", " 5 Left"),
+            MB_BAND_METERS("_6l", " 6 Left"),
+            MB_BAND_METERS("_7l", " 7 Left"),
+
+            MB_BAND_METERS("_0r", " 0 Right"),
+            MB_BAND_METERS("_1r", " 1 Right"),
+            MB_BAND_METERS("_2r", " 2 Right"),
+            MB_BAND_METERS("_3r", " 3 Right"),
+            MB_BAND_METERS("_4r", " 4 Right"),
+            MB_BAND_METERS("_5r", " 5 Right"),
+            MB_BAND_METERS("_6r", " 6 Right"),
+            MB_BAND_METERS("_7r", " 7 Right"),
+
             PORTS_END
         };
 
@@ -291,23 +348,41 @@ namespace lsp
             MB_SPLIT("_6r", " 6 Right", 1.0f, 3984.0f),
             MB_SPLIT("_7r", " 7 Right", 0.0f, 10000.0f),
 
-            MB_STEREO_BAND("_0l", " 0 Left", 0, 8, 10.0f, 40.0f),
-            MB_STEREO_BAND("_1l", " 1 Left", 1, 8, 40.0f, 100.0f),
-            MB_STEREO_BAND("_2l", " 2 Left", 2, 8, 100.0f, 252.0f),
-            MB_STEREO_BAND("_3l", " 3 Left", 3, 8, 252.0f, 632.0f),
-            MB_STEREO_BAND("_4l", " 4 Left", 4, 8, 632.0f, 1587.0f),
-            MB_STEREO_BAND("_5l", " 5 Left", 5, 8, 1587.0f, 3984.0f),
-            MB_STEREO_BAND("_6l", " 6 Left", 6, 8, 3984.0f, 10000.0f),
-            MB_STEREO_BAND("_7l", " 7 Left", 7, 8, 10000.0f, 20000.0f),
+            MB_SPLIT_BAND("_0l", " 0 Left", 0, 8, 10.0f, 40.0f),
+            MB_SPLIT_BAND("_1l", " 1 Left", 1, 8, 40.0f, 100.0f),
+            MB_SPLIT_BAND("_2l", " 2 Left", 2, 8, 100.0f, 252.0f),
+            MB_SPLIT_BAND("_3l", " 3 Left", 3, 8, 252.0f, 632.0f),
+            MB_SPLIT_BAND("_4l", " 4 Left", 4, 8, 632.0f, 1587.0f),
+            MB_SPLIT_BAND("_5l", " 5 Left", 5, 8, 1587.0f, 3984.0f),
+            MB_SPLIT_BAND("_6l", " 6 Left", 6, 8, 3984.0f, 10000.0f),
+            MB_SPLIT_BAND("_7l", " 7 Left", 7, 8, 10000.0f, 20000.0f),
 
-            MB_STEREO_BAND("_0r", " 0 Right", 0, 8, 10.0f, 40.0f),
-            MB_STEREO_BAND("_1r", " 1 Right", 1, 8, 40.0f, 100.0f),
-            MB_STEREO_BAND("_2r", " 2 Right", 2, 8, 100.0f, 252.0f),
-            MB_STEREO_BAND("_3r", " 3 Right", 3, 8, 252.0f, 632.0f),
-            MB_STEREO_BAND("_4r", " 4 Right", 4, 8, 632.0f, 1587.0f),
-            MB_STEREO_BAND("_5r", " 5 Right", 5, 8, 1587.0f, 3984.0f),
-            MB_STEREO_BAND("_6r", " 6 Right", 6, 8, 3984.0f, 10000.0f),
-            MB_STEREO_BAND("_7r", " 7 Right", 7, 8, 10000.0f, 20000.0f),
+            MB_SPLIT_BAND("_0r", " 0 Right", 0, 8, 10.0f, 40.0f),
+            MB_SPLIT_BAND("_1r", " 1 Right", 1, 8, 40.0f, 100.0f),
+            MB_SPLIT_BAND("_2r", " 2 Right", 2, 8, 100.0f, 252.0f),
+            MB_SPLIT_BAND("_3r", " 3 Right", 3, 8, 252.0f, 632.0f),
+            MB_SPLIT_BAND("_4r", " 4 Right", 4, 8, 632.0f, 1587.0f),
+            MB_SPLIT_BAND("_5r", " 5 Right", 5, 8, 1587.0f, 3984.0f),
+            MB_SPLIT_BAND("_6r", " 6 Right", 6, 8, 3984.0f, 10000.0f),
+            MB_SPLIT_BAND("_7r", " 7 Right", 7, 8, 10000.0f, 20000.0f),
+
+            MB_BAND_METERS("_0l", " 0 Left"),
+            MB_BAND_METERS("_1l", " 1 Left"),
+            MB_BAND_METERS("_2l", " 2 Left"),
+            MB_BAND_METERS("_3l", " 3 Left"),
+            MB_BAND_METERS("_4l", " 4 Left"),
+            MB_BAND_METERS("_5l", " 5 Left"),
+            MB_BAND_METERS("_6l", " 6 Left"),
+            MB_BAND_METERS("_7l", " 7 Left"),
+
+            MB_BAND_METERS("_0r", " 0 Right"),
+            MB_BAND_METERS("_1r", " 1 Right"),
+            MB_BAND_METERS("_2r", " 2 Right"),
+            MB_BAND_METERS("_3r", " 3 Right"),
+            MB_BAND_METERS("_4r", " 4 Right"),
+            MB_BAND_METERS("_5r", " 5 Right"),
+            MB_BAND_METERS("_6r", " 6 Right"),
+            MB_BAND_METERS("_7r", " 7 Right"),
 
             PORTS_END
         };
@@ -339,23 +414,41 @@ namespace lsp
             MB_SPLIT("_6s", " 6 Side", 1.0f, 3984.0f),
             MB_SPLIT("_7s", " 7 Side", 0.0f, 10000.0f),
 
-            MB_STEREO_BAND("_0m", " 0 Mid", 0, 8, 10.0f, 40.0f),
-            MB_STEREO_BAND("_1m", " 1 Mid", 1, 8, 40.0f, 100.0f),
-            MB_STEREO_BAND("_2m", " 2 Mid", 2, 8, 100.0f, 252.0f),
-            MB_STEREO_BAND("_3m", " 3 Mid", 3, 8, 252.0f, 632.0f),
-            MB_STEREO_BAND("_4m", " 4 Mid", 4, 8, 632.0f, 1587.0f),
-            MB_STEREO_BAND("_5m", " 5 Mid", 5, 8, 1587.0f, 3984.0f),
-            MB_STEREO_BAND("_6m", " 6 Mid", 6, 8, 3984.0f, 10000.0f),
-            MB_STEREO_BAND("_7m", " 7 Mid", 7, 8, 10000.0f, 20000.0f),
+            MB_SPLIT_BAND("_0m", " 0 Mid", 0, 8, 10.0f, 40.0f),
+            MB_SPLIT_BAND("_1m", " 1 Mid", 1, 8, 40.0f, 100.0f),
+            MB_SPLIT_BAND("_2m", " 2 Mid", 2, 8, 100.0f, 252.0f),
+            MB_SPLIT_BAND("_3m", " 3 Mid", 3, 8, 252.0f, 632.0f),
+            MB_SPLIT_BAND("_4m", " 4 Mid", 4, 8, 632.0f, 1587.0f),
+            MB_SPLIT_BAND("_5m", " 5 Mid", 5, 8, 1587.0f, 3984.0f),
+            MB_SPLIT_BAND("_6m", " 6 Mid", 6, 8, 3984.0f, 10000.0f),
+            MB_SPLIT_BAND("_7m", " 7 Mid", 7, 8, 10000.0f, 20000.0f),
 
-            MB_STEREO_BAND("_0s", " 0 Side", 0, 8, 10.0f, 40.0f),
-            MB_STEREO_BAND("_1s", " 1 Side", 1, 8, 40.0f, 100.0f),
-            MB_STEREO_BAND("_2s", " 2 Side", 2, 8, 100.0f, 252.0f),
-            MB_STEREO_BAND("_3s", " 3 Side", 3, 8, 252.0f, 632.0f),
-            MB_STEREO_BAND("_4s", " 4 Side", 4, 8, 632.0f, 1587.0f),
-            MB_STEREO_BAND("_5s", " 5 Side", 5, 8, 1587.0f, 3984.0f),
-            MB_STEREO_BAND("_6s", " 6 Side", 6, 8, 3984.0f, 10000.0f),
-            MB_STEREO_BAND("_7s", " 7 Side", 7, 8, 10000.0f, 20000.0f),
+            MB_SPLIT_BAND("_0s", " 0 Side", 0, 8, 10.0f, 40.0f),
+            MB_SPLIT_BAND("_1s", " 1 Side", 1, 8, 40.0f, 100.0f),
+            MB_SPLIT_BAND("_2s", " 2 Side", 2, 8, 100.0f, 252.0f),
+            MB_SPLIT_BAND("_3s", " 3 Side", 3, 8, 252.0f, 632.0f),
+            MB_SPLIT_BAND("_4s", " 4 Side", 4, 8, 632.0f, 1587.0f),
+            MB_SPLIT_BAND("_5s", " 5 Side", 5, 8, 1587.0f, 3984.0f),
+            MB_SPLIT_BAND("_6s", " 6 Side", 6, 8, 3984.0f, 10000.0f),
+            MB_SPLIT_BAND("_7s", " 7 Side", 7, 8, 10000.0f, 20000.0f),
+
+            MB_BAND_METERS("_0m", " 0 Mid"),
+            MB_BAND_METERS("_1m", " 1 Mid"),
+            MB_BAND_METERS("_2m", " 2 Mid"),
+            MB_BAND_METERS("_3m", " 3 Mid"),
+            MB_BAND_METERS("_4m", " 4 Mid"),
+            MB_BAND_METERS("_5m", " 5 Mid"),
+            MB_BAND_METERS("_6m", " 6 Mid"),
+            MB_BAND_METERS("_7m", " 7 Mid"),
+
+            MB_BAND_METERS("_0s", " 0 Side"),
+            MB_BAND_METERS("_1s", " 1 Side"),
+            MB_BAND_METERS("_2s", " 2 Side"),
+            MB_BAND_METERS("_3s", " 3 Side"),
+            MB_BAND_METERS("_4s", " 4 Side"),
+            MB_BAND_METERS("_5s", " 5 Side"),
+            MB_BAND_METERS("_6s", " 6 Side"),
+            MB_BAND_METERS("_7s", " 7 Side"),
 
             PORTS_END
         };
@@ -386,6 +479,15 @@ namespace lsp
             MB_SC_MONO_BAND("_6", " 6", 6, 8, 3984.0f, 10000.0f),
             MB_SC_MONO_BAND("_7", " 7", 7, 8, 10000.0f, 20000.0f),
 
+            MB_BAND_METERS("_0", " 0"),
+            MB_BAND_METERS("_1", " 1"),
+            MB_BAND_METERS("_2", " 2"),
+            MB_BAND_METERS("_3", " 3"),
+            MB_BAND_METERS("_4", " 4"),
+            MB_BAND_METERS("_5", " 5"),
+            MB_BAND_METERS("_6", " 6"),
+            MB_BAND_METERS("_7", " 7"),
+
             PORTS_END
         };
 
@@ -394,7 +496,7 @@ namespace lsp
             PORTS_STEREO_PLUGIN,
             PORTS_STEREO_SIDECHAIN,
             MB_COMMON(exp_sc_bands),
-            MB_CHANNEL("", ""),
+            MB_STEREO_CHANNEL,
             MB_FFT_METERS("_l", " Left"),
             MB_CHANNEL_METERS("_l", " Left"),
             MB_FFT_METERS("_r", " Right"),
@@ -417,6 +519,24 @@ namespace lsp
             MB_SC_STEREO_BAND("_6", " 6", 6, 8, 3984.0f, 10000.0f),
             MB_SC_STEREO_BAND("_7", " 7", 7, 8, 10000.0f, 20000.0f),
 
+            MB_BAND_METERS("_0l", " 0 Left"),
+            MB_BAND_METERS("_1l", " 1 Left"),
+            MB_BAND_METERS("_2l", " 2 Left"),
+            MB_BAND_METERS("_3l", " 3 Left"),
+            MB_BAND_METERS("_4l", " 4 Left"),
+            MB_BAND_METERS("_5l", " 5 Left"),
+            MB_BAND_METERS("_6l", " 6 Left"),
+            MB_BAND_METERS("_7l", " 7 Left"),
+
+            MB_BAND_METERS("_0r", " 0 Right"),
+            MB_BAND_METERS("_1r", " 1 Right"),
+            MB_BAND_METERS("_2r", " 2 Right"),
+            MB_BAND_METERS("_3r", " 3 Right"),
+            MB_BAND_METERS("_4r", " 4 Right"),
+            MB_BAND_METERS("_5r", " 5 Right"),
+            MB_BAND_METERS("_6r", " 6 Right"),
+            MB_BAND_METERS("_7r", " 7 Right"),
+
             PORTS_END
         };
 
@@ -448,23 +568,41 @@ namespace lsp
             MB_SPLIT("_6r", " 6 Right", 1.0f, 3984.0f),
             MB_SPLIT("_7r", " 7 Right", 0.0f, 10000.0f),
 
-            MB_SC_STEREO_BAND("_0l", " 0 Left", 0, 8, 10.0f, 40.0f),
-            MB_SC_STEREO_BAND("_1l", " 1 Left", 1, 8, 40.0f, 100.0f),
-            MB_SC_STEREO_BAND("_2l", " 2 Left", 2, 8, 100.0f, 252.0f),
-            MB_SC_STEREO_BAND("_3l", " 3 Left", 3, 8, 252.0f, 632.0f),
-            MB_SC_STEREO_BAND("_4l", " 4 Left", 4, 8, 632.0f, 1587.0f),
-            MB_SC_STEREO_BAND("_5l", " 5 Left", 5, 8, 1587.0f, 3984.0f),
-            MB_SC_STEREO_BAND("_6l", " 6 Left", 6, 8, 3984.0f, 10000.0f),
-            MB_SC_STEREO_BAND("_7l", " 7 Left", 7, 8, 10000.0f, 20000.0f),
+            MB_SC_SPLIT_BAND("_0l", " 0 Left", 0, 8, 10.0f, 40.0f),
+            MB_SC_SPLIT_BAND("_1l", " 1 Left", 1, 8, 40.0f, 100.0f),
+            MB_SC_SPLIT_BAND("_2l", " 2 Left", 2, 8, 100.0f, 252.0f),
+            MB_SC_SPLIT_BAND("_3l", " 3 Left", 3, 8, 252.0f, 632.0f),
+            MB_SC_SPLIT_BAND("_4l", " 4 Left", 4, 8, 632.0f, 1587.0f),
+            MB_SC_SPLIT_BAND("_5l", " 5 Left", 5, 8, 1587.0f, 3984.0f),
+            MB_SC_SPLIT_BAND("_6l", " 6 Left", 6, 8, 3984.0f, 10000.0f),
+            MB_SC_SPLIT_BAND("_7l", " 7 Left", 7, 8, 10000.0f, 20000.0f),
 
-            MB_SC_STEREO_BAND("_0r", " 0 Right", 0, 8, 10.0f, 40.0f),
-            MB_SC_STEREO_BAND("_1r", " 1 Right", 1, 8, 40.0f, 100.0f),
-            MB_SC_STEREO_BAND("_2r", " 2 Right", 2, 8, 100.0f, 252.0f),
-            MB_SC_STEREO_BAND("_3r", " 3 Right", 3, 8, 252.0f, 632.0f),
-            MB_SC_STEREO_BAND("_4r", " 4 Right", 4, 8, 632.0f, 1587.0f),
-            MB_SC_STEREO_BAND("_5r", " 5 Right", 5, 8, 1587.0f, 3984.0f),
-            MB_SC_STEREO_BAND("_6r", " 6 Right", 6, 8, 3984.0f, 10000.0f),
-            MB_SC_STEREO_BAND("_7r", " 7 Right", 7, 8, 10000.0f, 20000.0f),
+            MB_SC_SPLIT_BAND("_0r", " 0 Right", 0, 8, 10.0f, 40.0f),
+            MB_SC_SPLIT_BAND("_1r", " 1 Right", 1, 8, 40.0f, 100.0f),
+            MB_SC_SPLIT_BAND("_2r", " 2 Right", 2, 8, 100.0f, 252.0f),
+            MB_SC_SPLIT_BAND("_3r", " 3 Right", 3, 8, 252.0f, 632.0f),
+            MB_SC_SPLIT_BAND("_4r", " 4 Right", 4, 8, 632.0f, 1587.0f),
+            MB_SC_SPLIT_BAND("_5r", " 5 Right", 5, 8, 1587.0f, 3984.0f),
+            MB_SC_SPLIT_BAND("_6r", " 6 Right", 6, 8, 3984.0f, 10000.0f),
+            MB_SC_SPLIT_BAND("_7r", " 7 Right", 7, 8, 10000.0f, 20000.0f),
+
+            MB_BAND_METERS("_0l", " 0 Left"),
+            MB_BAND_METERS("_1l", " 1 Left"),
+            MB_BAND_METERS("_2l", " 2 Left"),
+            MB_BAND_METERS("_3l", " 3 Left"),
+            MB_BAND_METERS("_4l", " 4 Left"),
+            MB_BAND_METERS("_5l", " 5 Left"),
+            MB_BAND_METERS("_6l", " 6 Left"),
+            MB_BAND_METERS("_7l", " 7 Left"),
+
+            MB_BAND_METERS("_0r", " 0 Right"),
+            MB_BAND_METERS("_1r", " 1 Right"),
+            MB_BAND_METERS("_2r", " 2 Right"),
+            MB_BAND_METERS("_3r", " 3 Right"),
+            MB_BAND_METERS("_4r", " 4 Right"),
+            MB_BAND_METERS("_5r", " 5 Right"),
+            MB_BAND_METERS("_6r", " 6 Right"),
+            MB_BAND_METERS("_7r", " 7 Right"),
 
             PORTS_END
         };
@@ -497,23 +635,41 @@ namespace lsp
             MB_SPLIT("_6s", " 6 Side", 1.0f, 3984.0f),
             MB_SPLIT("_7s", " 7 Side", 0.0f, 10000.0f),
 
-            MB_SC_STEREO_BAND("_0m", " 0 Mid", 0, 8, 10.0f, 40.0f),
-            MB_SC_STEREO_BAND("_1m", " 1 Mid", 1, 8, 40.0f, 100.0f),
-            MB_SC_STEREO_BAND("_2m", " 2 Mid", 2, 8, 100.0f, 252.0f),
-            MB_SC_STEREO_BAND("_3m", " 3 Mid", 3, 8, 252.0f, 632.0f),
-            MB_SC_STEREO_BAND("_4m", " 4 Mid", 4, 8, 632.0f, 1587.0f),
-            MB_SC_STEREO_BAND("_5m", " 5 Mid", 5, 8, 1587.0f, 3984.0f),
-            MB_SC_STEREO_BAND("_6m", " 6 Mid", 6, 8, 3984.0f, 10000.0f),
-            MB_SC_STEREO_BAND("_7m", " 7 Mid", 7, 8, 10000.0f, 20000.0f),
+            MB_SC_SPLIT_BAND("_0m", " 0 Mid", 0, 8, 10.0f, 40.0f),
+            MB_SC_SPLIT_BAND("_1m", " 1 Mid", 1, 8, 40.0f, 100.0f),
+            MB_SC_SPLIT_BAND("_2m", " 2 Mid", 2, 8, 100.0f, 252.0f),
+            MB_SC_SPLIT_BAND("_3m", " 3 Mid", 3, 8, 252.0f, 632.0f),
+            MB_SC_SPLIT_BAND("_4m", " 4 Mid", 4, 8, 632.0f, 1587.0f),
+            MB_SC_SPLIT_BAND("_5m", " 5 Mid", 5, 8, 1587.0f, 3984.0f),
+            MB_SC_SPLIT_BAND("_6m", " 6 Mid", 6, 8, 3984.0f, 10000.0f),
+            MB_SC_SPLIT_BAND("_7m", " 7 Mid", 7, 8, 10000.0f, 20000.0f),
 
-            MB_SC_STEREO_BAND("_0s", " 0 Side", 0, 8, 10.0f, 40.0f),
-            MB_SC_STEREO_BAND("_1s", " 1 Side", 1, 8, 40.0f, 100.0f),
-            MB_SC_STEREO_BAND("_2s", " 2 Side", 2, 8, 100.0f, 252.0f),
-            MB_SC_STEREO_BAND("_3s", " 3 Side", 3, 8, 252.0f, 632.0f),
-            MB_SC_STEREO_BAND("_4s", " 4 Side", 4, 8, 632.0f, 1587.0f),
-            MB_SC_STEREO_BAND("_5s", " 5 Side", 5, 8, 1587.0f, 3984.0f),
-            MB_SC_STEREO_BAND("_6s", " 6 Side", 6, 8, 3984.0f, 10000.0f),
-            MB_SC_STEREO_BAND("_7s", " 7 Side", 7, 8, 10000.0f, 20000.0f),
+            MB_SC_SPLIT_BAND("_0s", " 0 Side", 0, 8, 10.0f, 40.0f),
+            MB_SC_SPLIT_BAND("_1s", " 1 Side", 1, 8, 40.0f, 100.0f),
+            MB_SC_SPLIT_BAND("_2s", " 2 Side", 2, 8, 100.0f, 252.0f),
+            MB_SC_SPLIT_BAND("_3s", " 3 Side", 3, 8, 252.0f, 632.0f),
+            MB_SC_SPLIT_BAND("_4s", " 4 Side", 4, 8, 632.0f, 1587.0f),
+            MB_SC_SPLIT_BAND("_5s", " 5 Side", 5, 8, 1587.0f, 3984.0f),
+            MB_SC_SPLIT_BAND("_6s", " 6 Side", 6, 8, 3984.0f, 10000.0f),
+            MB_SC_SPLIT_BAND("_7s", " 7 Side", 7, 8, 10000.0f, 20000.0f),
+
+            MB_BAND_METERS("_0m", " 0 Mid"),
+            MB_BAND_METERS("_1m", " 1 Mid"),
+            MB_BAND_METERS("_2m", " 2 Mid"),
+            MB_BAND_METERS("_3m", " 3 Mid"),
+            MB_BAND_METERS("_4m", " 4 Mid"),
+            MB_BAND_METERS("_5m", " 5 Mid"),
+            MB_BAND_METERS("_6m", " 6 Mid"),
+            MB_BAND_METERS("_7m", " 7 Mid"),
+
+            MB_BAND_METERS("_0s", " 0 Side"),
+            MB_BAND_METERS("_1s", " 1 Side"),
+            MB_BAND_METERS("_2s", " 2 Side"),
+            MB_BAND_METERS("_3s", " 3 Side"),
+            MB_BAND_METERS("_4s", " 4 Side"),
+            MB_BAND_METERS("_5s", " 5 Side"),
+            MB_BAND_METERS("_6s", " 6 Side"),
+            MB_BAND_METERS("_7s", " 7 Side"),
 
             PORTS_END
         };
