@@ -75,6 +75,7 @@ namespace lsp
                     plug::IPort        *pType;          // Filter type
                     plug::IPort        *pMode;          // Filter mode
                     plug::IPort        *pFreq;          // Filter frequency
+                    plug::IPort        *pWidth;         // Filter width
                     plug::IPort        *pSlope;         // Filter slope
                     plug::IPort        *pSolo;          // Solo port
                     plug::IPort        *pMute;          // Mute port
@@ -147,6 +148,8 @@ namespace lsp
                 static inline dspu::equalizer_mode_t get_eq_mode(ssize_t mode);
                 static inline void  decode_filter(size_t *ftype, size_t *slope, size_t mode);
                 static inline bool  adjust_gain(size_t filter_type);
+                static bool         filter_inspect_can_be_enabled(eq_channel_t *c, eq_filter_t *f);
+                static bool         filter_has_width(size_t type);
 
             protected:
                 void                destroy_state();
@@ -156,8 +159,6 @@ namespace lsp
                 static void         dump_filter(dspu::IStateDumper *v, const eq_filter_t *f);
                 static void         dump_filter_params(dspu::IStateDumper *v, const char *id, const dspu::filter_params_t *fp);
 
-                bool                filter_inspect_can_be_enabled(eq_channel_t *c, eq_filter_t *f);
-
             public:
                 explicit para_equalizer(const meta::plugin_t *metadata, size_t filters, size_t mode);
                 virtual ~para_equalizer() override;
